#line 157 "ped.nw"
#include "PathControl.h"
#include <string.h>
#include <sys/stat.h>
#line 168 "ped.nw"
void  PathControl::init(const string& initPath, char sep)
{
   _sep = sep;
   const char*  ip = initPath.c_str();
   const char*  curr = ip;
   const char*  pSep;
   while ((pSep = strchr(curr, _sep)))
   {
      _path.push_back(string(curr, (string::size_type)(pSep - curr)));
      curr = pSep + 1;
   }
   _path.push_back(string(curr));
}
#line 189 "ped.nw"
string  PathControl::sPath() const
{
   string  sp;
   const   string  sSep((size_t)1, _sep);
   string          currSep; // empty first, from 2nd loop becomes ':'
   for (vector<string>::const_iterator  i = _path.begin(), e = _path.end();
        i != e;  ++i)
   {
      sp += currSep;
      sp += *i;
      currSep = sSep;
   }
   return sp;
}
#line 218 "ped.nw"
void  PathControl::uniqueFilter()
{
   set<string>   seenAlready;
   unsigned      n = _path.size();
   vector<bool>  mask((size_t)n, true);
   bool          anyRemove = false;
   for (unsigned i = 0;  i < n;  ++i)
   {
      const string& comp = _path[i];
      pair<set<string>::iterator, bool>  insertion = seenAlready.insert(comp);
      if (!insertion.second)
      {
         mask[i] = false;
         anyRemove = true;
      }
   }
   if (anyRemove)
   {
      filterOut(mask);
   }
}
#line 303 "ped.nw"
bool isDirectory(const char* dirName)
{
   struct stat statBuf;
   int  rc = stat(dirName, &statBuf);
   bool isDir = (rc == 0) && (S_ISDIR(statBuf.st_mode));
   return isDir;
}
#line 263 "ped.nw"
void  PathControl::existsFilter()
{
   unsigned      n = _path.size();
   vector<bool>  mask((size_t)n, true);
   bool          anyRemove = false;
   for (unsigned i = 0;  i < n;  ++i)
   {
      const char*   s = _path[i].c_str();
      if (s[0] == '/')
      {
         string        dir = _path[i];
         const char*   dblslash = strstr(s, "//");
         if (dblslash)
         {
            dir = string(s, dblslash - s);
         }
         else
         {
            unsigned  l = dir.length();
            if ((l > 1) && (dir[l - 1] == '/'))
            {
               dir.resize(l - 1);
            }
         }
         if (!isDirectory(dir.c_str()))
         {
            mask[i] = isDirectory(dir.c_str());
            anyRemove = true;
         }
      }
   }
   if (anyRemove)
   {
      filterOut(mask);
   }
}
#line 317 "ped.nw"
void  PathControl::filterOut(const vector<bool>& mask)
{
   vector<string>::iterator      iNew = _path.begin();
   vector<bool>::const_iterator  iMask = mask.begin();
   for (vector<string>::const_iterator  iOld = _path.begin(), e = _path.end();
        iOld != e;  ++iOld, ++iMask)
   {
      if (*iMask)
      {
         *iNew = *iOld;
         ++iNew;
      }
   }
   _path.erase(iNew, _path.end());
}
