#line 1302 "ped.nw"
#include "Ped.h"
#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/clist.h>
#include <gtk--/dialog.h>
#include <gtk--/entry.h>
#include <gtk--/fileselection.h>
#include <gtk--/main.h>
#include <gtk--/menu.h>
#include <gtk--/menuitem.h>
#include <gtk--/optionmenu.h>
#include <gtk--/scrolledwindow.h>
using SigC::slot;
#line 616 "ped.nw"
class PedButton : public Gtk::Button
{
 public:
   PedButton(Ped& pd, void (Ped::*f)(), const string& label) :
      Gtk::Button(label),
      _ped(pd),
      _f(f)
      {}
   virtual ~PedButton() {}
 private:
   void clicked_impl() { (_ped.*_f)(); }
   Ped&  _ped;
   void (Ped::*_f)();
};
#line 474 "ped.nw"
Ped::Ped(const string& initPath, bool initUnique, bool initExists) :
   _fsDeleted(false),
   _edDlgDeleted(false),
   _pathCtl(initPath),
   _clist(0),
   _vbox(0),
   _fSel(0),
   _editDialog(0)
{

   // static const gchar* titles[] = {"Directories", 0 };
   _clist =  manage(new Gtk::CList(1));
   _clist->set_selection_mode(GTK_SELECTION_BROWSE);
   _clist->set_column_width (0, 150);
   Gtk::ScrolledWindow*  scrolledWin =  manage(new Gtk::ScrolledWindow());
   scrolledWin->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   scrolledWin->add(*_clist);

   Gtk::HBox*  hbox = manage(new Gtk::HBox(false, 2));
   Gtk::HBox*  hboxLeft = manage(new Gtk::HBox(true, 1));
   Gtk::HBox*  hboxRight = manage(new Gtk::HBox(true, 1));
   _vbox = manage(new Gtk::VBox(FALSE, 5));

   _vbox->set_border_width(5);
   _vbox->pack_start(*scrolledWin, TRUE, TRUE, 0);
   _vbox->pack_start(*hbox,        FALSE, TRUE, 0);

   addButton(hboxLeft, &(Ped::add), "Add...");
   _editButton = addButton(hboxLeft, &Ped::edit, "Edit...");
   _cutButton  = addButton(hboxLeft, &Ped::cut,  "Cut");
   addButton(hboxLeft, &Ped::paste, "Paste");
   hbox->pack_start(*hboxLeft);

   hbox->pack_start( *createOptionMenu(), false );

   addButton(hboxRight, &Ped::unique, "Unique");
   addButton(hboxRight, &Ped::exists, "Exists");
   hbox->pack_start(*hboxRight);

   if (initUnique)
   {
      _pathCtl.uniqueFilter();
   }
   if (initExists)
   {
      _pathCtl.existsFilter();
   }
   
   ctl2list();
   sync();
}
#line 532 "ped.nw"
Gtk::Widget*
Ped::addButton(Gtk::HBox* box, void (Ped::*f)(), const char* label)
{
   // cerr << "Ped::addButton: label=" << label << endl;
   PedButton*  b = manage(new PedButton(*this, f, label));
   box->pack_start(*b);
   return b;
}
#line 543 "ped.nw"
Ped::~Ped()
{
   delete _fSel;
   delete _editDialog;
}
#line 552 "ped.nw"
Gtk::Widget*  Ped::box() {return _vbox;}
#line 563 "ped.nw"
void Ped::ctl2list()
{
   _clist->clear();
   const char*  sRow[2] = {0, 0};
   Gtk::CList::RowList&  rows = _clist->rows();
   for (vector<string>::const_iterator
           i = _pathCtl.path().begin(), e = _pathCtl.path().end(); i != e; ++i)
   {
      sRow[0] = (*i).c_str();
      rows.push_back(sRow);
   }
}
#line 585 "ped.nw"
void Ped::list2ctl()
{
   const Gtk::CList::RowList&  rows = _clist->rows();
   unsigned                    nRows = rows.size();
   vector<string>              p((size_t)nRows, string(""));
   for (unsigned  i = 0;  i != nRows;  ++i)
   {
      const Gtk::CList::Row&   row = rows[i];
      const Gtk::CList::Cell&  cell = row[0];
      p[i] = cell.get_text();
      // cout << "[" << i << "] " << p[i] << endl;
   }
   _pathCtl.init(p);
}
#line 644 "ped.nw"
Gtk::OptionMenu*  Ped::createOptionMenu()
{
   _positionMenu = manage(new Gtk::Menu);
   Gtk::OptionMenu* optMenu = manage(new Gtk::OptionMenu);
   optMenu->set_usize(72, -1);
   Gtk::MenuItem*  before = manage(new Gtk::MenuItem("Before"));
   _afterItem  = manage(new Gtk::MenuItem("After"));
   Gtk::Menu::MenuList&  items = _positionMenu->items();
   items.push_back(*before);
   items.push_back(*_afterItem);
   // _positionMenu->set_active(1); // After as default
   // _positionMenu->set_title("Position"); // After as default
   optMenu->set_menu(_positionMenu);
   return optMenu;
}
#line 674 "ped.nw"
bool  Ped::isAfter() const
{
   Gtk::MenuItem const*  active = _positionMenu->get_active();
   return(active == _afterItem);
}
#line 692 "ped.nw"
int  Ped::selectedRow() const
{
   /* const */ Gtk::CList::SelectionList&  selections =  _clist->selection();
   int  rowNum = -1;
   if (!selections.empty())
   {
      const Gtk::CList::Row&  row = *(selections.begin());
      rowNum = row.get_row_num();
   }
   return  rowNum;
}
#line 718 "ped.nw"
unsigned  Ped::targetRow() const
{
   /* const */ Gtk::CList::SelectionList&  selections =  _clist->selection();
   gint  rowNum = 0;
   if (!selections.empty())
   {
      const Gtk::CList::Row&  row = *(selections.begin());
      rowNum = row.get_row_num() + (isAfter() ? 1 : 0);
   }
   return  rowNum;
}
#line 747 "ped.nw"
class PedFileSelect : public Gtk::FileSelection
{
 public:
   
#line 769 "ped.nw"
PedFileSelect(Ped& ped, const string& label) :
   Gtk::FileSelection(label),
   _ped(ped)
{
   hide_fileop_buttons();
   GtkFileSelection*  _gFSel = gtkobj();
   GtkWidget*         fileList = _gFSel->file_list;
   GtkWidget*         flScrollWin = fileList->parent;
   // gtk_widget_hide(fileList);
   gtk_widget_hide(flScrollWin);
   get_ok_button()->clicked.connect(slot(this, &PedFileSelect::ok));
   get_cancel_button()->clicked.connect(slot(this, &PedFileSelect::cancel));
}
#line 751 "ped.nw"
   virtual ~PedFileSelect() {}
 private:
    gint delete_event_impl(GdkEventAny*)
    {
       _ped._fsDeleted = true;
       return 0;
    }
    void  ok() { _ped.addDirName(get_filename());  hide(); }
    void  cancel() { hide(); }
    Ped&  _ped;
};
#line 785 "ped.nw"
void Ped::add()
{
   if (_fsDeleted)
   {
      delete _fSel;
      _fSel = 0;
      _fsDeleted = false;
   }
   if (_fSel == 0)
   {
      _fSel = new PedFileSelect(*this, "Add New Directory");
   }
   _fSel->show();
   Gtk::Main::grab_add(*_fSel);
}
#line 813 "ped.nw"
void Ped::addDirName(const string& fsDirname)
{
   string       dirName(fsDirname);
   unsigned     sz = dirName.size();
   const char*  s = dirName.c_str();
   if ((sz >= 2) && (s[sz-1] == '/') && (s[sz-2] != '/'))
   {
      dirName.resize(sz - 1);
   }
   char const*  sRow[2] = {dirName.c_str(), 0};
   _clist->insert_row(targetRow(), sRow);
   Gtk::Main::grab_remove(*_fSel);
   sync();
   changeNotify();
}
#line 844 "ped.nw"
class PedDialog : public Gtk::Dialog
{
 public:
   PedDialog(Ped& ped) : _ped(ped) {}
   virtual ~PedDialog() {}
 private:
    gint delete_event_impl(GdkEventAny*)
    {
       _ped._edDlgDeleted = true;
       return 0;
    }
   Ped&  _ped;
};
#line 863 "ped.nw"
void Ped::edit()
{
   if (_edDlgDeleted)
   {
      delete _editDialog;
      _editDialog = 0;
      _edDlgDeleted = false;
   }
   if (_editDialog == 0)
   {
      _editDialog = new PedDialog(*this);
      _editDialog->set_title("Edit Path Component");
      _editDialog->set_usize(500, 96);


      Gtk::HBox*  subHbox = manage(new Gtk::HBox(false, 5));
      _editEntry = manage(new Gtk::Entry);

      Gtk::VBox*  vbox = _editDialog->get_vbox();
      subHbox->pack_start(*_editEntry, true, true, 5);
      vbox->pack_start(*subHbox, true, true, 5);

      Gtk::HBox*  hbox = _editDialog->get_action_area();
      hbox->set_spacing(64);
      addButton(hbox, &Ped::editOK, "OK");
      addButton(hbox, &Ped::editCancel, "Cancel");
   }
   int                         rowNum = selectedRow();
   const Gtk::CList::RowList&  rows = _clist->rows();
   const Gtk::CList::Row&      row = rows[rowNum];
   const Gtk::CList::Cell&     cell = row[0];
   string                      currComp = cell.get_text();
   _editEntry->set_text(currComp);
   _editDialog->show_all();
   Gtk::Main::grab_add(*_editDialog);
}
#line 902 "ped.nw"
void Ped::editOK()
{
   string                newComp = _editEntry->get_text();
   int                   rowNum = selectedRow();
   _clist->set_text(rowNum, 0, newComp);
   Gtk::Main::grab_remove(*_editDialog);
   _editDialog->hide();
}
#line 913 "ped.nw"
void Ped::editCancel()
{
   Gtk::Main::grab_remove(*_editDialog);
   _editDialog->hide();
}
#line 938 "ped.nw"
void Ped::cut()
{
   int  rowNum = selectedRow();
   if (rowNum >= 0)
   {
      const Gtk::CList::RowList&  rows = _clist->rows();
      const Gtk::CList::Row&      row = rows[rowNum];
      const Gtk::CList::Cell&     cell = row[0];
      _clip = cell.get_text();
      _clist->remove_row(rowNum);
      sync();
      changeNotify();
   }
}
#line 961 "ped.nw"
void Ped::paste()
{
   char const*  sRow[2] = {_clip.c_str(), 0};
   _clist->insert_row(targetRow(), sRow);
   sync();
   changeNotify();
}
#line 982 "ped.nw"
void Ped::unique()
{
   list2ctl();
   const unsigned  oldSize = _pathCtl.size();
   _pathCtl.uniqueFilter();
   if (oldSize != _pathCtl.size())
   {
      ctl2list();
      changeNotify();
   }
}
#line 1000 "ped.nw"
void Ped::exists()
{
   list2ctl();
   const unsigned  oldSize = _pathCtl.size();
   _pathCtl.existsFilter();
   if (oldSize != _pathCtl.size())
   {
      ctl2list();
      changeNotify();
   }
}
#line 1020 "ped.nw"
void Ped::sync()
{
   bool anySelection = (selectedRow() >= 0);
   _editButton->set_sensitive(anySelection);
   _cutButton->set_sensitive(anySelection);
}
